package com.perfectech.cts.activities;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.net.Uri;
import android.provider.Settings;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.support.v4.app.TaskStackBuilder;
import android.support.v4.app.NavUtils;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.TextView;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Locale;

import android.os.Handler;

import com.perfectech.cts.DBLogs;
import com.perfectech.cts.DateConvertClass;
import com.perfectech.cts.JobClass;
import com.perfectech.cts.JobListTextViewListAdapter;
import com.perfectech.cts.syncClasses.JobTracking;
import com.perfectech.cts.objects.Jobs;
import com.perfectech.cts.R;
import com.perfectech.cts.CTSDBHelper;
import com.perfectech.cts.syncClasses.dataSyncService;

public class MainActivity extends AppCompatActivity {

    private static final String sDBName = "CTS.db";
    private static final int iVersionVal = 6;
    private static String sAndroidID = "";
    private static String sUserID = "";
    private static ArrayList<ArrayList<String>> jobArray;
    private Handler oHandler = new Handler();
    private static String sClosedDays = "2"; //days of closed jobs to show

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        final MainActivity mainActivity = this;
        final Intent jobsIntent = new Intent(this, JobsCenter.class);
        final Intent dvirIntent = new Intent(this, DVIRActivity.class);
        final Intent profileIntent = new Intent(this, UserProfile.class);
        final Intent messageIntent = new Intent(this, UserMessages.class);

        SharedPreferences prefs = getSharedPreferences("CTSSharedPrefs",
                Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        String sLoggedInUserCompany = prefs.getString("LoggedInUserCompany", "No Value");
        sUserID = prefs.getString("UserID", "No Value");

        String sCurrentVersion = prefs.getString("CTSVersion", "No Value");
        TextView otvUserName = (TextView) findViewById(R.id.tvUserName);
        TextView otvCopyright = (TextView) findViewById(R.id.tvCopyright);
        String sVersionText = otvCopyright.getText().toString();
        otvUserName.setText(sLoggedInUserCompany.substring(0,sLoggedInUserCompany.indexOf('-')));

        // need to replace the %RELNUM% with the actual rel version
        sVersionText = sVersionText.replace("RELNUM", sCurrentVersion);
        otvCopyright.setText(sVersionText);

        Button btnJobsCenter = (Button) findViewById(R.id.btnDVIREntry);
        btnJobsCenter.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                finish();

                // Use TaskStackBuilder to build the back stack and get the PendingIntent
                TaskStackBuilder.create(mainActivity)
                        // add all of DetailsActivity's parents to the stack,
                        // followed by DetailsActivity itself
                        .addNextIntentWithParentStack(jobsIntent)
                        .startActivities();
            }
        });

        Button btnDVIRCenter = (Button) findViewById(R.id.btnDVIR);
        btnDVIRCenter.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                finish();

                // Use TaskStackBuilder to build the back stack and get the PendingIntent
                TaskStackBuilder.create(mainActivity)
                        // add all of DetailsActivity's parents to the stack,
                        // followed by DetailsActivity itself
                        .addNextIntentWithParentStack(dvirIntent)
                        .startActivities();
            }
        });

        Button btnUserProfile = (Button) findViewById(R.id.btnProfile);
        btnUserProfile.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                finish();

                // Use TaskStackBuilder to build the back stack and get the PendingIntent
                TaskStackBuilder.create(mainActivity)
                        // add all of DetailsActivity's parents to the stack,
                        // followed by DetailsActivity itself
                        .addNextIntentWithParentStack(profileIntent)
                        .startActivities();
            }
        });

        Button btnUserMessage = (Button) findViewById(R.id.btnDVIRImageCapture);
        btnUserMessage.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                finish();
                // Use TaskStackBuilder to build the back stack and get the PendingIntent
                TaskStackBuilder.create(mainActivity)
                        // add all of DetailsActivity's parents to the stack,
                        // followed by DetailsActivity itself
                        .addNextIntentWithParentStack(messageIntent)
                        .startActivities();
            }
        });

        oHandler.post(timedTask);
        // setup the background sync service
        try {
            Intent iSvc2 = new Intent(this, dataSyncService.class);
            startService(iSvc2);
        } catch (Exception e) {
            Log.d("Error: startDataSyncSvc()", e.toString());
        }

        JobClass job = new JobClass();
        job.oContext = getApplicationContext();
        if (job.activeJob()) {
            try {
                Intent iSvc3 = new Intent(this, JobTracking.class);
                startService(iSvc3);
            } catch (Exception e) {
                Log.d("Error: startJobTrackingSvc()", e.toString());
            }
        }

    }

    @Override
    public void onResume() {
        super.onResume();
        updateUI();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_logout) {
            //Enable this to stop processes on logout
            JobClass job = new JobClass();
            job.oContext = getApplicationContext();
            if (!job.activeJob()) {
                stopService(new Intent(this, dataSyncService.class));
                stopService(new Intent(this, JobTracking.class));
                finish();
                Intent i = new Intent(getApplicationContext(), CTSActivity.class);
                startActivity(i);
            } else {
                String sTitle = getResources().getString(R.string.main_activity_active_job_error);
                String sMessage = getResources().getString(R.string.main_activity_active_job_error_message);
                errorDialog(sTitle, sMessage);
            }
            return true;
        }
        if (id == R.id.action_help) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "MainActivity");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_apphelp) {
            Intent newIntent = new Intent(Intent.ACTION_VIEW,
                    Uri.parse("http://www.commercialtrackingsolutions.com/TIS_Ad_App_User_Guide.htm"));
            startActivity(newIntent);
            return true;
        }
        if (id == R.id.action_webhelp) {
            Intent newIntent = new Intent(Intent.ACTION_VIEW,
                    Uri.parse("http://www.commercialtrackingsolutions.com/TIS_Web_user_guide.htm"));
            startActivity(newIntent);
            return true;
        }
        if (id == R.id.action_about) {
            Intent i = new Intent(getApplicationContext(), AboutActivity.class);
            startActivity(i);
            return true;
        }
        if (id == R.id.action_icon_meanings) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "IconMeanings");
            startActivity(i);
            return true;
        }

        switch (item.getItemId()) {

            case android.R.id.home:
                Intent upIntent = NavUtils.getParentActivityIntent(this);
                if (NavUtils.shouldUpRecreateTask(this, upIntent)) {
                    // This activity is NOT part of this app's task, so create a new task
                    // when navigating up, with a synthesized back stack.
                    TaskStackBuilder.create(this)
                            // Add all of this activity's parents to the back stack
                            .addNextIntentWithParentStack(upIntent)
                                    // Navigate up to the closest parent
                            .startActivities();
                } else {
                    // This activity is part of this app's task, so simply
                    // navigate up to the logical parent activity.
                    NavUtils.navigateUpTo(this, upIntent);
                }
                return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private Runnable timedTask = new Runnable() {
        @Override
        public void run() {
            // TODO Auto-generated method stub
            updateUI();
            oHandler.postDelayed(timedTask, 20000);
        }
    };


    /*****************************************************************************************/
    /* This method will get the number of available messages */

    /**
     * *************************************************************************************
     */
    private Integer getMsgCount() {
        Integer iRtnVal = 0;
        try {
            CTSDBHelper oDB = new CTSDBHelper(getApplicationContext(), sDBName,
                    null, iVersionVal);
            ArrayList<String> deviceArray;
            try {
                oDB.createDataBase();
                oDB.openDataBase();
                // select the counts of unread messages for the logged in user
                String sSQL = "select count(*) ";
                sSQL += "from Messages ";
                sSQL += "where MsgStatus = 'New' ";
                sSQL += "and MsgID <> '' ";
                deviceArray = oDB.getList(sSQL, null);
            } catch (IOException ioe) {
                DBLogs.insert("Error: " + ioe.toString(),
                        "TISActivity.getMsgCount:createDB",
                        getApplicationContext(), sDBName, iVersionVal);
                throw new Error("Unable to create database");
            } finally {
                oDB.close();
            }

            sAndroidID = Settings.System.getString(getContentResolver(),
                    Settings.System.ANDROID_ID);
            String[] deviceInfoArray;

            deviceInfoArray = new String[deviceArray.size()];
            deviceInfoArray = deviceArray.toArray(deviceInfoArray);

            if (deviceArray.size() > 0) {
                iRtnVal = Integer.parseInt(deviceInfoArray[0]);
            } else {
                iRtnVal = 0;
            }
        } catch (Exception e) {
            DBLogs.insert("Error: " + e.toString(),
                    "CTSActivity.getMessageCount", getApplicationContext(),
                    sDBName, iVersionVal);
            iRtnVal = 0;
        }

        Log.d("msgCount: ", iRtnVal.toString());
        return iRtnVal;

    }
    /*****************************************************************************************/
	/* This method will get the number of available jobs */

    /**
     * *************************************************************************************
     */
    private Integer getJobCount() {
        Integer iRtnVal = 0;
        try {
            CTSDBHelper oDB = new CTSDBHelper(getApplicationContext(), sDBName,
                    null, iVersionVal);
            try {
                oDB.createDataBase();
            } catch (IOException ioe) {
                DBLogs.insert("Error: " + ioe.toString(),
                        "CTSActivity.getJOBCount:createDB",
                        getApplicationContext(), sDBName, iVersionVal);
                throw new Error("Unable to create database");
            }

            oDB.openDataBase();

            sAndroidID = Settings.System.getString(getContentResolver(),
                    Settings.System.ANDROID_ID);

            // select the counts of jobs for the logged in user
            String sSQL = "select count(*) ";
            sSQL += "from Jobs ";
            sSQL += "where job_status_id in (1,8)";

            ArrayList<String> deviceArray = oDB.getList(sSQL, null);

            // closes the database
            try {
                oDB.close();
            } catch (Throwable e) {
                DBLogs.insert("Error: " + e.toString(),
                        "CTSActivity.getJobCount():closeDB",
                        getApplicationContext(), sDBName, iVersionVal);
                e.printStackTrace();
            }

            String[] deviceInfoArray;

            deviceInfoArray = new String[deviceArray.size()];
            deviceInfoArray = deviceArray.toArray(deviceInfoArray);

            if (deviceArray.size() > 0) {
                iRtnVal = Integer.parseInt(deviceInfoArray[0]);
            } else {
                iRtnVal = 0;
            }
        } catch (Exception e) {
            DBLogs.insert("Error: " + e.toString(),
                    "CTSActivity.getJobCount", getApplicationContext(),
                    sDBName, iVersionVal);
            iRtnVal = 0;
        }

        Log.d("jobCount: ", iRtnVal.toString());
        return iRtnVal;

    }


    /*****************************************************************************************/
	/* This method will get the number of available jobs */

    /**
     * *************************************************************************************
     */
    private void getJobsFromDB() {
        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = getApplicationContext();

        CTSDBHelper oDB = new CTSDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
            oDB.openDataBase();
            String sLanguage = Locale.getDefault().getLanguage();
            String sSQL = "";
            switch (sLanguage) {
                case "en":
                    sSQL = "select A.job_id, A.date_time, A.job_status_id, B.name_en, A.assigned_id, A.manifest_num";
                    sSQL += ", A.container_num, A.delivery_address_line1, A.delivery_address_line2, A.delivery_city";
                    sSQL += ", A.delivery_state, A.delivery_postal_code, A.pickup_address_line1, A.pickup_address_line2";
                    sSQL += ", A.pickup_city, A.pickup_state, A.pickup_postal_code, A.username";
                    sSQL += ", A.importer_id, A.importer_name, A.job_num, A.pickup_date_time, A.delivery_date_time";
                    sSQL += ", A.ref_note from jobs as A join job_status_type as B on A.job_status_id = B.id";
                    sSQL += " where A.job_status_id not in (6,9,11)";
                    break;
                case "es":
                    sSQL = "select A.job_id, A.date_time, A.job_status_id, B.name_sp, A.assigned_id, A.manifest_num";
                    sSQL += ", A.container_num, A.delivery_address_line1, A.delivery_address_line2, A.delivery_city";
                    sSQL += ", A.delivery_state, A.delivery_postal_code, A.pickup_address_line1, A.pickup_address_line2";
                    sSQL += ", A.pickup_city, A.pickup_state, A.pickup_postal_code, A.username";
                    sSQL += ", A.importer_id, A.importer_name, A.job_num, A.pickup_date_time, A.delivery_date_time";
                    sSQL += ", A.ref_note from jobs as A join job_status_type as B on A.job_status_id = B.id";
                    sSQL += " where A.job_status_id not in (6,9,11)";
                    break;
                default:
                    sSQL = "select A.job_id, A.date_time, A.job_status_id, B.name_en, A.assigned_id, A.manifest_num";
                    sSQL += ", A.container_num, A.delivery_address_line1, A.delivery_address_line2, A.delivery_city";
                    sSQL += ", A.delivery_state, A.delivery_postal_code, A.pickup_address_line1, A.pickup_address_line2";
                    sSQL += ", A.pickup_city, A.pickup_state, A.pickup_postal_code, A.username";
                    sSQL += ", A.importer_id, A.importer_name, A.job_num, A.pickup_date_time, A.delivery_date_time";
                    sSQL += ", A.ref_note from jobs as A join job_status_type as B on A.job_status_id = B.id";
                    sSQL += " where A.job_status_id not in (6,9,11)";
            }
            jobArray = oDB.selectRecordsFromDBList(sSQL, null);
            Log.d("sSQL", sSQL);
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        } finally {
            oDB.close();
        }

        ListView jobs = (ListView) this
                .findViewById(R.id.lvOpenJobs);

        SimpleAdapter sd;
        ArrayList<Jobs> allJobs = new ArrayList<Jobs>();
        Integer iSize = jobArray.size();
        Log.d("iSize", iSize.toString());
        if (iSize > 0) {
            for (ArrayList<String> sValue : jobArray) {
                Jobs jobObj = new Jobs();
                jobObj.job_id = sValue.get(0);
                //Convert the GMT database time to local device time
                jobObj.date_time = oDateConvertClass.GMTToLocal(sValue.get(1));
                jobObj.job_status_id = sValue.get(2);
                jobObj.status_name = sValue.get(3);
                jobObj.assigned_id = sValue.get(4);
                jobObj.importer_id = sValue.get(18);
                jobObj.importer_name = sValue.get(19);
                jobObj.manifest_num = sValue.get(5);
                jobObj.container_num = sValue.get(6);
                jobObj.delivery_address_line1 = sValue.get(7);
                jobObj.delivery_address_line2 = sValue.get(8);
                jobObj.delivery_city = sValue.get(9);
                jobObj.delivery_state = sValue.get(10);
                jobObj.delivery_postal_code = sValue.get(11);
                jobObj.pickup_address_line1 = sValue.get(12);
                jobObj.pickup_address_line2 = sValue.get(13);
                jobObj.pickup_city = sValue.get(14);
                jobObj.pickup_state = sValue.get(15);
                jobObj.pickup_postal_code = sValue.get(16);
                jobObj.username = sValue.get(17);
                jobObj.job_num = sValue.get(20);
                jobObj.pickup_date_time = oDateConvertClass.GMTToLocal(sValue.get(21));
                jobObj.delivery_date_time = oDateConvertClass.GMTToLocal(sValue.get(22));
                jobObj.ref_note = sValue.get(23);
                allJobs.add(jobObj);
            }

            jobs.setAdapter(new JobListTextViewListAdapter(this, R.layout.jobs_row, allJobs));
            jobs.setOnItemClickListener(new ListView.OnItemClickListener() {
                public void onItemClick(AdapterView<?> parent, View view,
                                        int position, long id) {
                    try {
                        // get the selected row's ticket number
                        Integer iCounter = 0;
                        for (ArrayList<String> sValue : jobArray) {
                            if (position == iCounter) {
                                Log.d("SelectedJobID", sValue.get(0));
                                selectJobs(sValue.get(0));
                            }
                            iCounter++;
                        }
                    } catch (Exception e) {
                        Log.d("Error:", e.toString());
                    }
                }
            });

        } else {
            // No open JObs found
            Log.d("getJobs", "No Jobs");
            String[] openTicketsArray = new String[]{getResources().getString(R.string.main_activity_no_jobs)};
            jobs.setAdapter(new ArrayAdapter<String>(this,
                    android.R.layout.simple_list_item_1, openTicketsArray));
        }

    }
    /********************************************************************************/
	/* Handle the selection of a job */

    /**
     * ****************************************************************************
     */
    private void selectJobs(final String sJobID) {
        // call the jobs_detail_activity and pass in the selected job_id
        Intent i = new Intent(getApplicationContext(),
                JobsDetailActivity.class);
        i.putExtra("JobID", sJobID);
        startActivity(i);
        //finish();
    }

    private void updateUI() {
        // get the current messages in the DB and display them in the list view
        getJobsFromDB();

        // need to get the number of messages
        TextView tvMsgBadge = (TextView) findViewById(R.id.msgCount);
        Integer iNumMsgs = getMsgCount();
        if (iNumMsgs > 0) {
            tvMsgBadge.setText(iNumMsgs.toString());
            tvMsgBadge.setVisibility(TextView.VISIBLE);
        } else {
            tvMsgBadge.setVisibility(TextView.INVISIBLE);
        }

        // need to get the number of pending jobs
        TextView tvJobBadge = (TextView) findViewById(R.id.jobCount);
        Integer iNumJobs = getJobCount();
        if (iNumJobs > 0) {
            tvJobBadge.setText(iNumJobs.toString());
            tvJobBadge.setVisibility(TextView.VISIBLE);
        } else {
            tvJobBadge.setVisibility(TextView.INVISIBLE);
        }
    }

    private void errorDialog(String sTitle, String sMessage) {
        TextView title = new TextView(this);
        title.setText(sTitle);
        title.setBackgroundColor(Color.RED);
        title.setPadding(10, 10, 10, 10);
        title.setGravity(Gravity.CENTER);
        title.setTextColor(Color.WHITE);
        title.setTextSize(20);
        new AlertDialog.Builder(this)
                .setCustomTitle(title)
                .setMessage(sMessage)
                .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        // close dialog and exit the app
                        dialog.dismiss();
                    }
                })
                .show();
    }
}
