package com.perfectech.cts.syncClasses;

import android.app.IntentService;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Environment;

import com.perfectech.cts.CommunicationsClass;
import com.perfectech.cts.DBLogs;
import com.perfectech.cts.DBParameters;
import com.perfectech.cts.DateConvertClass;
import com.perfectech.cts.objects.MessagesClass;

import java.io.File;

import com.perfectech.cts.webServices.SyncFiles;
import com.perfectech.cts.webServices.SyncJson;

/**
 * Created by jnissley on 8/1/15.
 * This is an Intent Service that does not run on the UI thread.
 * All for the syncs interact with the internet so we should run them on a background thread
 * The request are only processed one at a time.  There is only one thread.
 */
public class SyncIntentService extends IntentService {
    private static final String sDBName = "CTS.db";
    static int iVersionVal = 6;
    private Context oContext = null;
    private String sCurrentDateTime = "";
    private static boolean debug_mode = false;

    public SyncIntentService() {
        super("SyncIntentService");
    }

    @Override
    protected void onHandleIntent(Intent intent) {
        String sRecordType = intent.getStringExtra("RecType");
        String sID = intent.getStringExtra("ID"); //This is the ID of the sync table row
        String sRecID = intent.getStringExtra("RecID"); //This is the recid in the sync table
        sCurrentDateTime = getGMTDataTime();
        oContext = getApplicationContext();

        DBParameters oDBParameters = new DBParameters();
        try {
            String sDebugMode = oDBParameters.readParameter("DebugMode",
                    getApplicationContext(), sDBName, iVersionVal);
            if (sDebugMode.equals("1")) {
                debug_mode = true;
            } else {
                debug_mode = false;
            }
        } catch(Exception ex) {
            // To do
        }

        switch (sRecordType.toUpperCase()) {
            case "CN":  //New communication
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Sync New Comm Messages",
                            "SyncIntentService.onHandleIntent.CN",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncCommNew(sRecID, sID);
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.CN",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "DVIRN": //New DVIR
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Sync New DVIR",
                            "SyncIntentService.onHandleIntent.DVIRN",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncDVIRN(sRecID, sID);
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.DVIRN",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "DVIRC": //Completed DVIR
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Sync Completed DVIR",
                            "SyncIntentService.onHandleIntent.DVIRC",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncDVIRC(sRecID, sID);
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.DVIRC",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "PHOTO": //DVIR Photo
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Upload a DVIR Photo",
                            "SyncIntentService.onHandleIntent.syncDVIRFile",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncDVIRFile(sRecID, sID);
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.PHOTO",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "CU": // CU - Communication Update
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Update Comm Message",
                            "SyncIntentService.onHandleIntent.CU",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncCommStatus(sRecID, sID);
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.CU",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "UP": //UP - User Profile
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Update User Profile",
                            "SyncIntentService.onHandleIntent.UP",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncUserProfile(sRecID, sID);
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.UP",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "SIGPU":  //SIG signature file - PickUP
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Upload PickUp Signature File",
                            "SyncIntentService.onHandleIntent.SIGUP",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncSigFile(sRecID, sID, "PU");
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.SIGUP",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "SIGUSER":  //SIG signature file - PickUP
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Upload Driver Signature File",
                            "SyncIntentService.onHandleIntent.SIGUSER",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncSigFile(sRecID, sID, "USER");
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.SIGUSER",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "SIGDEL":  //SIG signature file - Delivery
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Delivery User Signature File",
                            "SyncIntentService.onHandleIntent.SIGDEL",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncSigFile(sRecID, sID, "DEL");
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.SIGDEL",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "SIGJOB":  //SIG job
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Job User Signature Names",
                            "SyncIntentService.onHandleIntent.SIGJOB",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncSigJob(sRecID, sID);
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.SIGJOB",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "UPHOTO":  //Sync User photo
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Upload User Photo",
                            "SyncIntentService.onHandleIntent.UPHOTO",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncUserFile(sRecID, sID);
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.UPHOTO",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "MESSAGE":  //Sync Messages on the server
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Sync Messages With Server",
                            "SyncIntentService.onHandleIntent.MESSAGE",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    SyncMessages.oContext = oContext;
                    SyncMessages.getMessages();
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.MESSAGE",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "JOBS":  //Sync Jobs on the server
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Sync Jobs With Server",
                            "SyncIntentService.onHandleIntent.JOBS",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    SyncJobs.oContext = oContext;
                    SyncJobs.getJobs();
                    SyncJobs.syncJobs();
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.JOBS",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "LOGS": //Sync the logs to the server
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Upload Device Logs To Server",
                            "SyncIntentService.onHandleIntent.LOGS",
                            oContext, sDBName,
                            iVersionVal);
                }
                try {
                    syncLogs();
                } catch (Exception ex) {
                    DBLogs.insert("Exception:" + ex.toString(),
                            "SyncIntentService.onHandleIntent.LOGS",
                            oContext, sDBName, iVersionVal);
                }
                break;
            case "JOBSTATUP": //Job Status Update
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Modify Job Status",
                            "SyncIntentService.onHandleIntent.JOBSTATUP",
                            oContext, sDBName,
                            iVersionVal);
                }
                    syncJobStatusUpdate(sRecID, sID);
                break;
            case "ORDERPU": //Pick Up Order
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Order Pickup",
                            "SyncIntentService.onHandleIntent.ORDERUP",
                            oContext, sDBName,
                            iVersionVal);
                }
                syncOrderPickup(sRecID, sID);
                break;
            case "JOBNOTEUP": //Job Status Update
                if (debug_mode == true) {
                    DBLogs.insert(
                            "Add Job Note",
                            "SyncIntentService.onHandleIntent.JOBNOTEUP",
                            oContext, sDBName,
                            iVersionVal);
                }
                syncJobNote(sRecID, sID);
                break;
            default:
                DBLogs.insert("Default case statement",
                        "SyncIntentService.onHandleIntent.default",
                        oContext, sDBName, iVersionVal);
                break;
        }
    }

    // Get the GMT time that will be inserted into any log messages
    private String getGMTDataTime() {
        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = oContext;
        return oDateConvertClass.GMTTimestamp();
    }

    /********************************************************************************/
	/* This method will sync new communications */
    /********************************************************************************/
    private void syncCommNew(String sInRecID, String sInID) {
        CommunicationsClass communicationClass = new CommunicationsClass();
        MessagesClass messagesClass = new MessagesClass();
        boolean bUpdated = false;
        communicationClass.oContext = oContext;
        messagesClass = communicationClass.read(sInRecID);

        try {
            SyncMessages.oContext = oContext;
            bUpdated = SyncMessages.sendMessage(messagesClass);
            if (bUpdated) {
                SyncTasks.update("1", sCurrentDateTime, sInID);
            } else {
                SyncTasks.updatequeued("0","",sInID);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            DBLogs.insert("Exception:" + ex.toString(),
                    "SyncIntentService.syncCommNew",
                    oContext, sDBName, iVersionVal);

        }
    }

    /********************************************************************************/
	/* This method will sync a new dvir */
    /********************************************************************************/
    private void syncDVIRN(String sInRecID, String sInID) {
        Boolean bUpdated = false;
        try {
            SyncDVIR SyncDVIR = new SyncDVIR();
            SyncDVIR.oContext = oContext;
            SyncDVIR.sDVIRNum = sInRecID;
            bUpdated = SyncDVIR.updNewDVIR();
            if (bUpdated) {
                SyncTasks.update("1", sCurrentDateTime, sInID);
            } else {
                SyncTasks.updatequeued("0","",sInID);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            DBLogs.insert("Exception:" + ex.toString(),
                    "SyncIntentService.syncDVIRN",
                    oContext, sDBName, iVersionVal);

        }
    }

    /********************************************************************************/
	/* This method will sync a completed dvir */
    /********************************************************************************/
    private void syncDVIRC(String sInRecID, String sInID) {
        Boolean bUpdated = false;
        try {
            SyncDVIR SyncDVIR = new SyncDVIR();
            SyncDVIR.oContext = oContext;
            SyncDVIR.sDVIRNum = sInRecID;
            bUpdated = SyncDVIR.updCompletedDVIR();
            if (bUpdated) {
                SyncTasks.update("1", sCurrentDateTime, sInID);
            } else {
                SyncTasks.updatequeued("0","",sInID);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            DBLogs.insert("Exception:" + ex.toString(),
                    "SyncIntentService.syncDVIRC",
                    oContext, sDBName, iVersionVal);

        }
    }

    /********************************************************************************/
	/* This method will send DVIR photos to the server                              */
    /********************************************************************************/

    private void syncDVIRFile(String sFileName, String sID) {
        DBParameters oDBParameters = new DBParameters();
        // WebServer Request URL
        String serverURL = oDBParameters.readParameter(
                "REST_BASE_URL", oContext, sDBName,
                iVersionVal);
        Boolean bUpdated = false;
        SyncFiles.oContext = oContext;
        File storageDir = new File(Environment.getExternalStorageDirectory() + "/cts");
        SyncFiles.fFilePath = storageDir;
        SyncFiles.sFileType = "dvirFile";
        SyncFiles.sWebService = serverURL + "UploadDVIRPic.php";
        SyncFiles.sFileName = sFileName;
        try {
            if (SyncFiles.uploadFile()) {
                //Mark the task as complete
                SyncTasks.update("1", sCurrentDateTime, sID);
            } else {
                SyncTasks.updatequeued("0","",sID);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            DBLogs.insert("Exception:" + ex.toString(),
                    "SyncIntentService.syncDVIRFile",
                    oContext, sDBName, iVersionVal);
        }
    }

    /********************************************************************************/
	/* This method will sync service tickets */
    /********************************************************************************/
    private void syncCommStatus(String sInRecID, String sInID) {
        CommunicationsClass communicationClass = new CommunicationsClass();
        MessagesClass messagesClass = new MessagesClass();
        boolean bUpdated = false;
        communicationClass.oContext = oContext;
        messagesClass = communicationClass.read(sInRecID);
        try {
            SyncMessages.oContext = oContext;
            bUpdated = SyncMessages.sendUpdStatus(messagesClass);
            if (bUpdated) {
                SyncTasks.update("1", sCurrentDateTime, sInID);
            } else {
                SyncTasks.updatequeued("0","",sInID);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            DBLogs.insert("Exception:" + ex.toString(),
                    "SyncIntentService.syncCommStatus",
                    oContext, sDBName, iVersionVal);

        }
    }

    /********************************************************************************/
	/* This method will sync the user profile tickets */
    /********************************************************************************/
    private void syncUserProfile(String sInRecID, String sInID) {
        Boolean bUpdated = false;
        try {
            bUpdated = SyncUsers.updProfile(sInRecID);
            if (bUpdated) {
                SyncTasks.update("1", sCurrentDateTime, sInID);
            } else {
                SyncTasks.updatequeued("0","",sInID);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            DBLogs.insert("Exception:" + ex.toString(),
                    "SyncIntentService.syncUserProfile",
                    oContext, sDBName, iVersionVal);

        }
    }

    /********************************************************************************/
	/* This method will send signature files to the server                          */
    /********************************************************************************/

    private void syncSigFile(String sFileName, String sID, String sType) {
        Boolean bUpdated = false;
        String sJobID = "";
        String sUserID = "";
        String sPUDelStatus = "";
        String sPrivateDir = "signatures";

        //Need to get the GPS data if this is a pickup or delivery
        if (sType.equals("PU")) {
            SyncFiles.sFileType = "signature";
            sPUDelStatus = "Picked up";
            sJobID = sFileName.substring(sFileName.lastIndexOf("_") + 1, sFileName.indexOf("."));
        }
        else if (sType.equals("DEL")) {
            SyncFiles.sFileType = "signature";
            sPUDelStatus = "Delivered";
            sJobID = sFileName.substring(sFileName.lastIndexOf("_") + 1, sFileName.indexOf("."));
        } else if (sType.equals("USER")) {
            SyncFiles.sFileType = "userfile"; //This means it will not get deleted in syncfiles upload
        }
        //Get the user id needed to update the Job Notes
        SharedPreferences prefs = getSharedPreferences("CTSSharedPrefs",
                Context.MODE_PRIVATE);
        //SharedPreferences.Editor editor = prefs.edit();
        sUserID = prefs.getString("UserID", "No Value");
        if (sJobID.length() > 0) {
            DateConvertClass oDateConvertClass = new DateConvertClass();
            oDateConvertClass.oContext = getApplicationContext();
            SyncFiles.oContext = getApplicationContext();
            String sCurrentDateTime = oDateConvertClass.GMTTimestamp();
            if (SyncFiles.updateStatus(sJobID, sUserID, sPUDelStatus, sCurrentDateTime)) {
                //Mark the task as complete
                SyncTasks.update("1", sCurrentDateTime, sID);
            }
        }
        else {
            SyncTasks.update("1", sCurrentDateTime, sID);
        }

        DBParameters oDBParameters = new DBParameters();
        // WebServer Request URL
        String serverURL = oDBParameters.readParameter(
                "REST_BASE_URL", oContext, sDBName,
                iVersionVal);
        File myDir = getDir(sPrivateDir, Context.MODE_PRIVATE);
        //File sigFile = new File(directory,sFileName);
        SyncFiles.sFileName = sFileName;
        SyncFiles.sWebService = serverURL + "UploadSigFiles.php";
        SyncFiles.fFilePath = myDir;
        SyncFiles.oContext = oContext;
        try {
            bUpdated = SyncFiles.uploadFile();
            if (bUpdated) {
                SyncTasks.update("1", sCurrentDateTime, sID);
            } else {
                SyncTasks.updatequeued("0","",sID);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            DBLogs.insert("Exception:" + ex.toString(),
                    "SyncIntentService.syncSigFile",
                    oContext, sDBName, iVersionVal);
        }
    }

    /********************************************************************************/
	/* This method will send user Photo to the server                               */
    /********************************************************************************/

    private void syncUserFile(String sFileName, String sID) {
        String sPrivateDir = "images";
        DBParameters oDBParameters = new DBParameters();
        // WebServer Request URL
        String serverURL = oDBParameters.readParameter(
                "REST_BASE_URL", oContext, sDBName,
                iVersionVal);
        SyncFiles.oContext = oContext;
        File myDir = getDir(sPrivateDir, Context.MODE_PRIVATE);
        SyncFiles.fFilePath = myDir;
        SyncFiles.sFileType = "userfile"; //This means it will not be deleted when uploaded
        SyncFiles.sWebService = serverURL + "UploadUserPic.php";
        SyncFiles.sFileName = sFileName;
        try {
            if (SyncFiles.uploadFile()) {
                //Mark the task as complete
                SyncTasks.update("1", sCurrentDateTime, sID);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            DBLogs.insert("Exception:" + ex.toString(),
                    "SyncIntentService.syncUserFile",
                    oContext, sDBName, iVersionVal);
        }
    }

    /*
    This method will sync the phone logs to the server to assist in troubleshooting
     */
    private void syncLogs() {
        CTSCommon.uploadLogs(oContext);
    }

    //Sync the job status update
    private void syncJobStatusUpdate(String sRecID, String sID) {
        SyncJson oSyncJobStatus = new SyncJson();
        oSyncJobStatus.oContext = oContext;
        oSyncJobStatus.sendString = sRecID;
        oSyncJobStatus.webserviceName = "updJobStatusWS.php";
        String sResponse = oSyncJobStatus.SyncData();
        if (sResponse.toUpperCase().equals("SUCCESS")) {
            //Mark the task as complete
            SyncTasks.update("1", sCurrentDateTime, sID);
        } else {
            //Set queued and queued time back to 0 so that sync process with pick it up
            SyncTasks.updatequeued("0","",sID);
        }
    }

    //Sync the order pickup status
    private void syncOrderPickup(String sRecID, String sID) {
        SyncJson oSyncOrderPickup = new SyncJson();
        oSyncOrderPickup.oContext = oContext;
        oSyncOrderPickup.sendString = sRecID;
        oSyncOrderPickup.webserviceName = "UpdPUDelStatusWS.php";
        String sResponse = oSyncOrderPickup.SyncData();
        if (sResponse.toUpperCase().equals("SUCCESS")) {
            //Mark the task as complete
            SyncTasks.update("1", sCurrentDateTime, sID);
        } else {
            //Set queued and queued time back to 0 so that sync process with pick it up
            SyncTasks.updatequeued("0","",sID);
        }
    }

    //Sync the job status update
    private void syncJobNote(String sRecID, String sID) {
        SyncJson oSyncJobNote = new SyncJson();
        oSyncJobNote.oContext = oContext;
        oSyncJobNote.sendString = sRecID;
        oSyncJobNote.webserviceName = "updJobNotesWS.php";
        String sResponse = oSyncJobNote.SyncData();
        if (sResponse.toUpperCase().equals("SUCCESS")) {
            //Mark the task as complete
            SyncTasks.update("1", sCurrentDateTime, sID);
        } else {
            //Set queued and queued time back to 0 so that sync process with pick it up
            SyncTasks.updatequeued("0","",sID);
        }
    }

    //Sync the job name update
    private void syncSigJob(String sRecID, String sID) {
        SyncJson oSyncSigJob = new SyncJson();
        oSyncSigJob.oContext = oContext;

        // need to build the JSON object to send
        String sSendString = SyncJobs.getSignatureNames(sRecID);

        oSyncSigJob.sendString = sSendString;  // JSON object
        oSyncSigJob.webserviceName = "updJobSig.php";
        String sResponse = oSyncSigJob.SyncData();
        if (sResponse.toUpperCase().equals("SUCCESS")) {
            //Mark the task as complete
            SyncTasks.update("1", sCurrentDateTime, sID);
        } else {
            //Set queued and queued time back to 0 so that sync process with pick it up
            SyncTasks.updatequeued("0","",sID);
        }
    }
}
