/********************************************************************************/
/* Class:	SyncTasks.java                                                      */
/* Date: 	May 1, 2012                                                         */
/* Version:	1.0.0                                                               */
/* Author:	Joe A Cairns                                                        */
/* Description                                                                  */
/*    This is the main class file for handling the SyncTasks entries            */
/*                                                                              */
/* Modification Log                                                             */
/*                                                                              */
/*           Copyright (c) 2015 PerfecTech                                      */
/********************************************************************************/
package com.perfectech.cts.syncClasses;

import java.io.IOException;
import java.util.ArrayList;

import android.content.ContentValues;
import android.content.Context;
import android.util.Log;

import com.perfectech.cts.DBLogs;
import com.perfectech.cts.CTSDBHelper;

/**
 * @author jcairns
 * 
 */
public class SyncTasks {
	/********************************************************************************/
	/* Define Global Variables */
	/********************************************************************************/
	private static final String sDBName = "CTS.db";
	private static final int iVersionVal = 6;
	public static Context oContext;

	public static String sDateTime;
	public static String sRecType;
	public static String sRecID;
	public static String sCompleted;
	public static String sDateTimeCompleted;
	public static String sQueued;
	public static String sDateTimeQueued;
	public static String sID;

	/********************************************************************************/
	/* read the SyncTasks table */
	/********************************************************************************/
	public static void read(String sInID) {
		String sSQL = "select _id, DateTime, RecType, RecID, Completed, DateTimeCompleted ";
		sSQL += "from SyncTasks ";
		sSQL += "where _id = '" + sInID + "' ";

		CTSDBHelper oDB = new CTSDBHelper(oContext, sDBName, null, iVersionVal);
		try {
			oDB.createDataBase();
		} catch (IOException ioe) {
			throw new Error("Unable to create database");
		}

		oDB.openDataBase();

		ArrayList<String> oArray = oDB.getList(sSQL, null);

		// closes the database
		try {
			oDB.close();
		} catch (Throwable e) {
			DBLogs.insert("Error: " + e.toString(), "SyncTasks.read", oContext,
					sDBName, iVersionVal);
			e.printStackTrace();
		}

		String[] oInfoArray = new String[oArray.size()];
		oInfoArray = oArray.toArray(oInfoArray);

		sID = oInfoArray[5];
		sDateTime = oInfoArray[4];
		sRecType = oInfoArray[3];
		sRecID = oInfoArray[2];
		sCompleted = oInfoArray[1];
		sDateTimeCompleted = oInfoArray[0];
	}

	/********************************************************************************/
	/* write the SyncTasks table */
	/********************************************************************************/
	public static long write() {
		long lRetValue = 0;
		// setup the values
		ContentValues oValues = new ContentValues();
		oValues.put("DateTime", sDateTime);
		oValues.put("RecType", sRecType);
		oValues.put("RecID", sRecID);
		oValues.put("Queued", sQueued);
		oValues.put("DateTimeQueued", sDateTimeQueued);
		oValues.put("Completed", sCompleted);
		oValues.put("DateTimeCompleted", sDateTimeCompleted);

		// insert the values to the table
		CTSDBHelper oDB = new CTSDBHelper(oContext, sDBName, null, iVersionVal);
		try {
			oDB.createDataBase();
		} catch (IOException ioe) {
			throw new Error("Unable to create database");
		}

		try {
			oDB.openDataBase();

			Log.d("insert", "Inserting SyncTasks");
			lRetValue = oDB.insertRecordsInDB("SyncTasks", null, oValues);
			oDB.close();

		} catch (Exception sqlEx) {
			Log.d("sqlEx:", sqlEx.toString());
			oDB.close();
		}
		return lRetValue;
	}

	/********************************************************************************/
	/* update the SyncTasks table for completed task*/
	/********************************************************************************/
	public static void update(String sInCompleted, String sInDateTimeCompleted, String sInID) {

		// setup the values
		ContentValues oValues = new ContentValues();
		oValues.put("Completed", sInCompleted);
		oValues.put("DateTimeCompleted", sInDateTimeCompleted);

		// insert the values to the table
		CTSDBHelper oDB = new CTSDBHelper(oContext, sDBName, null, iVersionVal);
		try {
			oDB.createDataBase();
		} catch (IOException ioe) {
			throw new Error("Unable to create database");
		}

		try {
			oDB.openDataBase();

			Log.d("update", "Inserting TRTransLog");
			String sWhere = "_id = '" + sInID + "'";

			oDB.updateRecordsInDB("SyncTasks", sWhere, oValues);
			oDB.close();

		} catch (Exception sqlEx) {
			Log.d("sqlEx:", sqlEx.toString());
			oDB.close();
		}
	}

	/********************************************************************************/
	/* update the SyncTasks table for queued task*/
	/********************************************************************************/
	public static void updatequeued(String sInQueued, String sInDateTimeQueued, String sInID) {

		// setup the values
		ContentValues oValues = new ContentValues();
		oValues.put("Queued", sInQueued);
		oValues.put("DateTimeQueued", sInDateTimeQueued);

		// insert the values to the table
		CTSDBHelper oDB = new CTSDBHelper(oContext, sDBName, null, iVersionVal);
		try {
			oDB.createDataBase();
		} catch (IOException ioe) {
			throw new Error("Unable to create database");
		}

		try {
			oDB.openDataBase();

			Log.d("update", "Inserting TRTransLog");
			String sWhere = "_id = '" + sInID + "'";

			oDB.updateRecordsInDB("SyncTasks", sWhere, oValues);
			oDB.close();

		} catch (Exception sqlEx) {
			Log.d("sqlEx:", sqlEx.toString());
			oDB.close();
		}
	}

	/********************************************************************************/
	/* get the next row to process */
	/********************************************************************************/
	public static void getNexRec() {
		String sSQL = "select min(_id) ";
		sSQL += "from SyncTasks ";
		sSQL += "where Queued = '0' ";
		sSQL += "order by _id ";

		CTSDBHelper oDB = new CTSDBHelper(oContext, sDBName, null, iVersionVal);
		try {
			oDB.createDataBase();
		} catch (IOException ioe) {
			throw new Error("Unable to create database");
		}
		
		oDB.openDataBase();
		
		ArrayList<String> oArray = oDB.getList(sSQL, null);

		// closes the database
		try {
			oDB.close();
		} catch (Throwable e) {
			DBLogs.insert("Error: " + e.toString(), "SyncTasks.read", oContext,
					sDBName, iVersionVal);
			e.printStackTrace();
		}

		String[] oInfoArray = new String[oArray.size()];
		oInfoArray = oArray.toArray(oInfoArray);

		sID = oInfoArray[0];

	}

	/********************************************************************************/
	/* get the next row to process */
	/********************************************************************************/
	public static int getNumRecs() {
		int iRtnVal = 0;

		try {
			String sSQL = "select count(*) ";
			sSQL += "from SyncTasks ";
			sSQL += "where Queued = '0' ";

			CTSDBHelper oDB = new CTSDBHelper(oContext, sDBName, null, iVersionVal);
			try {
				oDB.createDataBase();
			} catch (IOException ioe) {
				throw new Error("Unable to create database");
			}

			oDB.openDataBase();
			
			ArrayList<String> oArray = oDB.getList(sSQL, null);

			// close the database
			try {
				oDB.close();
			} catch (Throwable e) {
				DBLogs.insert("Error: " + e.toString(), "SyncTasks.read",
						oContext, sDBName, iVersionVal);
				e.printStackTrace();
			}

			String[] oInfoArray = new String[oArray.size()];
			oInfoArray = oArray.toArray(oInfoArray);

			iRtnVal = Integer.parseInt(oInfoArray[0]);

		} catch (Exception ex) {
			iRtnVal = 0;
			DBLogs.insert("SyncTasks.getNumRecs() Error:", ex.toString(), oContext, sDBName, iVersionVal);
		}

		return iRtnVal;
	}

}
